/**el archivo contiene las configuraciones correspondientes a la seleccion Table View
desde View*/

package workspace.Views;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.Point;
import java.util.ArrayList;
import javax.swing.ImageIcon;
import workspace.Main_Set.SimpleNode;
import workspace.Prepro.FilterNode;
import workspace.Prepro.IntervalNode;
import workspace.Prepro.UmbralNode;
import workspace.Source.SelectSourceNode;

public class TableViewNode extends SimpleNode{
    /**se esta forzando a variable*/
    private Class<?>[] clases;
    private ArrayList<SimpleNode> nodos;
    /**las siguientes banderas establecen si el objeto ya fue configurado
    o ejecutado*/
    public boolean configured = true;
    public boolean   executed = false;
    public String pathLogServer = "";
    public int tipoL;
    public int tSesion;
    
    public TableViewNode(Point center) {
        /**se establece centro, dimension*/
        this.setCenter(center);
        this.setDimension(new Dimension(80,60));
        /**se valida el nodo antecesor, los nodos le pueden anteceder*/
        this.clases = new Class<?>[4];
        this.clases[0] = SelectSourceNode.class;
        this.clases[1] = IntervalNode.class;
        this.clases[2] = UmbralNode.class;     
        this.clases[3] = FilterNode.class;
        /**se instancia un ArrayList en nodos*/
        this.nodos = new ArrayList();
    }
    
    public Class<?>[] getValidInputClasses(){
        /**retorna los antecesores...*/
        return this.clases;
    }
    
    public ArrayList<SimpleNode> getInputNodes(){
        /**se devuelven los valores de los nodos en un array forzado a tipo
        SimpleNode*/
        return this.nodos;
    }
    
    public Image getImage(){
        /**el icono*/
        return new ImageIcon(System.class.getResource("/media/table.png")).getImage();
    }
    
    public String getString(){
        /**retorna una cadena que lo identifica*/
        return "Table";
    }
    
    /**si el nodo puede ser conectado, entonces se aade un nodo*/
    public void connect(SimpleNode n){
        if(this.canConnect(n)){
            this.nodos.add(n);
        }
    }
    
    /**al desconectar el nodo se elimina la linea punteada y se coloca el color*/
    /**de borde por defecto*/
    public void disconnect(SimpleNode n){
        this.nodos.remove(n);
        this.setBorderColor(null);
    }
    
    //**retorna verdadero porque puede ser ejecutado*/
    public boolean canExecute() {
        return true;
    }
    
    public void execute() {
        // *** Polaris *** Johana ***
        
        // *** Finished ***
        if(this.getInputNodes().size()==1){
            String sourceClassName = this.getInputNodes().get(0).getClass().getCanonicalName();
            if (sourceClassName.equals(SelectSourceNode.class.getCanonicalName())){
                SelectSourceNode sn = (SelectSourceNode)this.getInputNodes().get(0);
                //***********************************************************
                /*aqui se instancia SS*/
                //***********************************************************               
                
                if(!sn.isConfigured()){
                    this.getParent().showError("The SS must be configured first");
                    //validacion para la badera ejecucion y el color del borde
                    this.setExecuted(false);
                    setBorderColor(null);
                } else if(!sn.isExecuted()){
                    this.getParent().showError("The SS must be executed first");
                    //validacion para la badera ejecucion y el color del borde
                    this.setExecuted(false);
                    setBorderColor(null);
                }  else{                    
                    this.setExecuted(true);
                    // *** Polaris *** Johana ***
                    long time = System.currentTimeMillis();
            //
                    fillRegistros freg= new fillRegistros();
                    if(freg.getRegs().size()==0){
                        this.setMsg("Error: Register don't load...");
                    }else{
                    ArrayList regis= freg.getRegs();
                    MostrarTabla mt= new MostrarTabla(regis,sn.getPath(),"Source");
                    /** centrar*/
                    mt.setLocationRelativeTo(mt.getParent());
                    /**Visible*/
                    mt.setVisible(true);                   
                    }
                    //
                    long executionTime = System.currentTimeMillis() - time;                       
            System.out.println("TableView tiempo: " + executionTime + " milisegundos");
                    // *** Finished ***
                    //pinta de verde si el nodo ya se ejecuto
                    setBorderColor(new Color(0,255,0));
                    getParent().repaint();
                }
            
            }else if(sourceClassName.equals(FilterNode.class.getCanonicalName())){
                
                FilterNode   fn = (FilterNode)this.getInputNodes().get(0);
                //***********************************************************
                /*aqui se instancia filter*/
                //***********************************************************
                this.setPathLogServer(fn.getPathLogServer());
                System.out.println(" From filter: " + this.getPathLogServer());
                
                if(!fn.isExecuted()){
                    this.getParent().showError("The filter Session must be executed first");
                    /**validacion para las baderas ejecucion, configuracion y el color del borde*/
                    this.setExecuted(false);
                    if(this.isConfigured() == false){
                        this.setConfigured(false);
                        setBorderColor(null);
                    } else{
                        this.setConfigured(true);
                        setBorderColor(new Color(0,0,255));
                    }
                } else{
                    
                    // *** Polaris *** Johana ***
                    if(fn.getTransCla().size()==0){
                        this.setMsg("Error: Discretize Data don't load...");
                    }else{
                        ArrayList regis= fn.getTransCla();
                        MostrarTabla mt= new MostrarTabla(regis,fn.getPathLogServer(),"Discretize Filter",fn.getIduracion(), fn.getIcantidad(), fn.getItamano());
                        /** centrar*/
                        mt.setLocationRelativeTo(mt.getParent());
                        /**Visible*/
                        mt.setVisible(true);                   
                    }                    
                    // *** Finished ***
                    /**cambia el valor de la bandera executed*/
                    this.setExecuted(true);
                    /**si el nodo fue configurado, el borde se torna verde*/
                    setBorderColor(new Color(0,255,0));
                    /**obtiene el contenedor y lo redibuja... si no se redibuja entonces
                     *al mostrarse graficamente presenta pixeles defectuosos*/
                    getParent().repaint();
                }
            }
            /**filternode close*/         
            else if (sourceClassName.equals(IntervalNode.class.getCanonicalName())){
                IntervalNode sn = (IntervalNode)this.getInputNodes().get(0);
                //***********************************************************
                /*aqui se instancia SS*/
                //***********************************************************               
                
                if(!sn.isConfigured()){
                    this.getParent().showError("The SS must be configured first");
                    //validacion para la badera ejecucion y el color del borde
                    this.setExecuted(false);
                    setBorderColor(null);
                } else if(!sn.isExecuted()){
                    this.getParent().showError("The SS must be executed first");
                    //validacion para la badera ejecucion y el color del borde
                    this.setExecuted(false);
                    setBorderColor(null);
                }  else{                    
                    this.setExecuted(true);
                    // *** Polaris *** Johana ***
                    long time = System.currentTimeMillis();
            //
                    fillSesiones fsesion= new fillSesiones(sn.getTipoL());
                    if(fsesion.getRegs().size()==0){
                        this.setMsg("Error: Sessions don't load...");
                    }else{
                    ArrayList sesiones= fsesion.getRegs();
                    int ts;
                    if(sn.getTipoL()==1 || sn.getTipoL()==3) ts=1;
                    else ts=2;
                    //
                    MostrarTabla mt= new MostrarTabla(sesiones,sn.getPathLogServer(),"Sessions",ts, sn.isMfr());
                    /** centrar*/
                    mt.setLocationRelativeTo(mt.getParent());
                    /**Visible*/
                    mt.setVisible(true);                   
                    }
                    //
                    long executionTime = System.currentTimeMillis() - time;                       
            System.out.println("TableView tiempo: " + executionTime + " milisegundos");
                    // *** Finished ***
                    //pinta de verde si el nodo ya se ejecuto
                    setBorderColor(new Color(0,255,0));
                    getParent().repaint();
                }
            
            }else if(sourceClassName.equals(UmbralNode.class.getCanonicalName())){
                
                UmbralNode   fn = (UmbralNode)this.getInputNodes().get(0);
                //***********************************************************
                /*aqui se instancia filter*/
                //***********************************************************
                this.setPathLogServer(fn.getPathLogServer());
                System.out.println(" From filter: " + this.getPathLogServer());
                
                if(!fn.isExecuted()){
                    this.getParent().showError("The filter Session must be executed first");
                    /**validacion para las baderas ejecucion, configuracion y el color del borde*/
                    this.setExecuted(false);
                    if(this.isConfigured() == false){
                        this.setConfigured(false);
                        setBorderColor(null);
                    } else{
                        this.setConfigured(true);
                        setBorderColor(new Color(0,0,255));
                    }
                } else{
                    
                    // *** Polaris *** Johana ***
                    long time = System.currentTimeMillis();
            //
                    fillSesiones fsesion= new fillSesiones(fn.getTipoL());
                    if(fsesion.getRegs().size()==0){
                        this.setMsg("Error: Sessions don't load...");
                    }else{
                    ArrayList sesiones= fsesion.getRegs();
                    int ts;
                    if(fn.getTipoL()==1 || fn.getTipoL()==3) ts=1;
                    else ts=2;
                    //
                    MostrarTabla mt= new MostrarTabla(sesiones,fn.getPathLogServer(),"Sessions",ts, fn.isMfr());
                    /** centrar*/
                    mt.setLocationRelativeTo(mt.getParent());
                    /**Visible*/
                    mt.setVisible(true);                   
                    }
                    //
                    long executionTime = System.currentTimeMillis() - time;                       
                    System.out.println("TableView tiempo: " + executionTime + " milisegundos");
                    // *** Finished ***
                    /**cambia el valor de la bandera executed*/
                    this.setExecuted(true);
                    /**si el nodo fue configurado, el borde se torna verde*/
                    setBorderColor(new Color(0,255,0));
                    /**obtiene el contenedor y lo redibuja... si no se redibuja entonces
                     *al mostrarse graficamente presenta pixeles defectuosos*/
                    getParent().repaint();
                }
            }
        }
        else{
            this.getParent().showError("First add a connection");
            /**validacion para la badera ejecucion y el color del borde*/
            this.setExecuted(false);
            setBorderColor(null);
            getParent().repaint();
        }
    }
 
    public String getPathLogServer() {
        return pathLogServer;
    }
    
    public void setPathLogServer(String pathLogServer) {
        this.pathLogServer = pathLogServer;
    }
    
    public int getTipoL() {
        return tipoL;
    }
    
    public void setTipoL(int tipoL) {
        this.tipoL = tipoL;
    }
    
    public int getTsesion() {
        return getTsesion();
    }
    
    public void setTsesion(int tSesion) {
        this.setTsesion(getTsesion());
    }
}
